/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementNode;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * This builder will build a datastructure that will collect all elements with 
 * an element that has a sml:targetRequired attribute set to true.  This builder is active in the phenic phase
 * @author sleeloy
 *
 */
public class TargetRequiredInstanceBuilder extends AbstractDataBuilder implements IDataBuilder {

	public static final String ID = SMLActivator.PLUGIN_ID + ".TargetRequiredInstanceBuilder";
	protected Set targetRequires;
    protected Map targetRequiredInstances = new HashMap();
    protected ElementNode currentTargetRequired; 
    protected boolean uriElementFound = false; 

	public Object getDataStructure() {
		return targetRequiredInstances;
	}

	public void startPhenics() {
		//need to determine the list of elements that have a targetelement attribute defined in the schema
		IDataBuilder targetRequiredBuilder = DataBuilderRegistry.instance().getDataStructureBuilder(TargetRequiredBuilder.ID);
		Map map = (Map)targetRequiredBuilder.getDataStructure();
		if (map != null){
			targetRequires = map.keySet();
		}
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		//no target elements defined in schema
		if (targetRequires == null) return;
		
		//check if the element is defined as having a target element attribute and the ref:targetRequired is true and 
		//the ref:type is set to true
		if ((targetRequires.contains(localName)) && ("true".equals(attributes.getValue(ISMLConstants.REF_ATTRIBUTE)))){
			currentTargetRequired = new ElementNode(localName,locator.getLineNumber(),locator.getColumnNumber());
		}
		//TODO: should check inheritance and substitution
		//check to see if the uri element is found
		else if ((currentTargetRequired != null) && (ISMLConstants.URI_ELEMENT.equals(qName))){
			uriElementFound = true;
		}
		else
			uriElementFound = false;
	}

	
	public void characters(char[] ch, int start, int len) throws SAXException {
		//extract the uri value
		if (uriElementFound){
			String target = new String(ch, start, len);
			if ((len == 0) || (target == null))
				target = ISMLConstants.SML_URI_VALUE_NULL; 
			targetRequiredInstances.put(currentTargetRequired, target);
		}
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {
		if ((uriElementFound) && (ISMLConstants.URI_ELEMENT.equals(qName))){
			uriElementFound = false;
		}
	}

	public byte getPhase() {
		return ISMLConstants.PHENICS_PHASE;
	}

}
