/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementNode;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * This builder will build a datastructure that will collect all elements with 
 * a target element.  This builder is active in the phenic phase
 * @author sleeloy
 *
 */
public class TargetTypeInstanceBuilder extends AbstractDataBuilder implements IDataBuilder {

	public static final String ID = SMLActivator.PLUGIN_ID + ".TargetTypeInstanceBuilder";
	protected Set targetTypes;
    Map targetTypeInstances = new HashMap();
    protected ElementNode currentTargetType; 
    protected boolean uriElementFound = false; 

	public Object getDataStructure() {
		return targetTypeInstances;
	}

	public void startPhenics() {
		//need to determine the list of elements that have a targetelement attribute defined in the schema
		IDataBuilder targetTypeBuilder = DataBuilderRegistry.instance().getDataStructureBuilder(TargetTypeBuilder.ID);
		Map map = (Map)targetTypeBuilder.getDataStructure();
		if (map != null){
			targetTypes = map.keySet();
		}
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		//no target elements defined in schema
		if (targetTypes == null) return;
		//check if the element is defined as having a target element attribute and the ref:type is set to true
		if ((targetTypes.contains(localName))&& ("true".equals(attributes.getValue(ISMLConstants.REF_ATTRIBUTE)))){
			currentTargetType = new ElementNode(localName,locator.getLineNumber(),locator.getColumnNumber());
		}
		//check to see if the uri element is found
		else if ((currentTargetType != null) && (ISMLConstants.URI_ELEMENT.equals(qName))){
			uriElementFound = true;
		}
		else
			uriElementFound = false;
	}

	
	public void characters(char[] ch, int start, int len) throws SAXException {
		//extract the uri value
		if (uriElementFound){
			String target = new String(ch, start, len);
			targetTypeInstances.put(currentTargetType, target);
		}
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {
		if ((uriElementFound) && (ISMLConstants.URI_ELEMENT.equals(qName))){
			uriElementFound = false;
		}
	}

	public byte getPhase() {
		return ISMLConstants.PHENICS_PHASE;
	}

}
