/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.reference;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;


/**
 * Represents an URI as specified by the grammer: 
 * {@link http://www.ietf.org/rfc/rfc3986.txt}
 * 
 * @author Ali Mehregani
 */
public class URIReference implements IReferenceExpression
{	
	/**
	 * The reference URI.
	 */
	private URI reference;
	
	/**
	 * The document reference
	 */
	private String documentReference;

	/**
	 * Indicates whether this reference has already been
	 * transformed.
	 */
	private boolean transformed;

	/**
	 * Constructor.  The reference must be a valid XPointer expression.
	 * 
	 * @param reference The string representing the XPointer expression 
	 * @throws URISyntaxException If the URI has an incorrect syntax
	 */
	public URIReference(String reference) throws URISyntaxException
	{
		if (reference == null)
			return;

		/* Trim the white spaces and take out line separators */
		reference = reference.trim();
		reference = reference.replaceAll("\\n", "");
		reference = reference.replaceAll("\\r", "");
	
		this.reference = new URI(reference);
	}

	
	/**
	 * The document reference is basically the <scheme>+<authority>+<path> 
	 * of the URI.
	 * 
	 * @return the documentAlias
	 */
	public String getDocumentReference()
	{
		if (!isTransformed())
			transform();
		
		if (documentReference == null)
		{
			StringBuffer docRef = new StringBuffer();
			docRef.append(reference.getScheme() == null ? IValidationConstants.EMPTY_STRING : reference.getScheme());
			docRef.append(reference.getAuthority() == null ? IValidationConstants.EMPTY_STRING : reference.getAuthority());
			docRef.append(reference.getPath() == null ? IValidationConstants.EMPTY_STRING : reference.getPath());
			documentReference = docRef.toString();
		}
		
		return documentReference;
	}


	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.reference.IReferenceExpression#isTransformed()
	 */
	public boolean isTransformed()
	{
		return transformed;
	}


	/**
	 * The transformation will be based on [RFC 3986]: {@link http://www.ietf.org/rfc/rfc3986.txt}.
	 * See section 5 for more details.
	 * 
	 * @see org.eclipse.cosmos.rm.validation.internal.reference.IReferenceExpression#transform()
	 */
	public void transform()
	{
		
		String scheme, authority, path, query;
		
		/* Determine the base URI */
//		TODO: Needs to be implemented
		
//
//
//	      if defined(R.scheme) then
//	         T.scheme    = R.scheme;
//	         T.authority = R.authority;
//	         T.path      = remove_dot_segments(R.path);
//	         T.query     = R.query;
//	      else
//	         if defined(R.authority) then
//	            T.authority = R.authority;
//	            T.path      = remove_dot_segments(R.path);
//	            T.query     = R.query;
//	         else
//	            if (R.path == "") then
//	               T.path = Base.path;
//	               if defined(R.query) then
//	                  T.query = R.query;
//	               else
//	                  T.query = Base.query;
//	               endif;
//	            else
//	               if (R.path starts-with "/") then
//	                  T.path = remove_dot_segments(R.path);
//	               else
//	                  T.path = merge(Base.path, R.path);
//	                  T.path = remove_dot_segments(T.path);
//	               endif;
//	               T.query = R.query;
//	            endif;
//	            T.authority = Base.authority;
//	         endif;
//	         T.scheme = Base.scheme;
//	      endif;
//
//	      T.fragment = R.fragment;
		
	}
}
